#!/usr/bin/env python
# -*- coding:utf-8 -*-

# Program: 分析orzdba的日志生成excel或图片
# Author : HH
# Date   : 2016-02-18

import sys
import os
import linecache
import xlwt
import time

reload(sys)
sys.setdefaultencoding('utf-8')

class OrzReport(object):
  '''
  通过分析生成excel报表或图片
  '''
  def __init__(self, file_name):
    '''
    初始化参数
    '''
    self.file_name = file_name
    # 默认样式
    self.default_style = xlwt.easyxf('''
      borders: left 1, right 1, top 1, bottom 1;
      align: horiz center;''')
    
    # 标题栏样式
    self.title_style = xlwt.easyxf('''
      pattern: pattern solid, fore_colour yellow;
      font: name Times New Roman, color-index black, bold on;
      borders: left 1, right 1, top 1, bottom 1;
      align: horiz center''',
      num_format_str='0,000.00')

    # 创建保存文件目录
    self.pic_dir = 'pic_{time}'.format(
      time = time.strftime('%Y-%m-%d_%H-%M-%S')
    )
    os.mkdir(self.pic_dir)

  def get_split_index_list(self, item_str, sep='|'):
    '''
    获得分割字符串的位置
    '''
    index_list = []
    index_start = 0
    for num in xrange(item_str.count(sep)):
      index_start = item_str.find(sep, index_start)
      index_list.append(index_start)
      index_start += 1
    return index_list
  
  def get_item_str_list(self, item_str, sep='|'):
    '''
    获得每个模块中每个指标。如：bytes模块的recv send元素
    '''
    item_str_list = item_str.split(sep)
    return item_str_list
    
  def get_module_list(self, module_name_str, separator_list=[]):
    '''
    获得每个项目的名字, 如：---innodb rows status---
                            ------threads------
    每个报表是根据模块为粒度的
    '''
    module_name_list = []
    for index, value in enumerate(separator_list, -1):
      if index == -1:
        module_name = module_name_str[:value]
        module_name_list.append(module_name)
      else:
        module_name = module_name_str[separator_list[index]+1:value]
        module_name_list.append(module_name)
    return module_name_list
  
  def build_data_list(self, data_list, module_item_list, sep='|'):
    '''
    获得数据数据列表。如:{'module':'bytes', 
                          'items' :[recv send],
                          'values':[ [1,2], [1,2] ]}
    '''
    data_dict_list = []
    # 创建模块和指标的基本信息
    for module, item_str in module_item_list:
      data_dict_base = {
        'module': module.strip(),
        'items' : item_str.strip().split(),
        'values': []
      }
      data_dict_list.append(data_dict_base)
  
    # 像每一个模块中添加数据
    for line in data_list:
      value_str_list = line.strip().strip(sep).split(sep)
      for index, value_str in enumerate(value_str_list, 0):
        value_list = value_str.strip().split()
        data_dict_list[index]['values'].append(value_list)
    return data_dict_list

  def add_item_value_to_module(self, col_dict, data_dict_list, pos=0):
    '''
    在模块中添加列
    '''
    for dict_index, data_dict in enumerate(data_dict_list, 0):
      # 插入item值
      data_dict_list[dict_index]['items'].insert(
        pos,
        ''.join(col_dict['items']))
      for value_index, value in enumerate(data_dict['values'], 0):
        data_dict_list[dict_index]['values'][value_index].insert(
          pos,
          ''.join(col_dict['values'][value_index]))
    return data_dict_list
  
  def get_col_max_length(self, data_list, item_list):
    '''
    获得数据每列最大值长度
    '''
    col_len = map(len, map(str, item_list))
    func = lambda x, y: y if y>x else x
    
    for row in data_list:
      row_len = map(len, map(str, row))
      col_len = map(func, col_len, row_len)
    return col_len
  
  def create_item_len(self, data_dict_list):
    '''
    计算出每一个指标需要的宽度
    '''
    for index, data_dict in enumerate(data_dict_list, 0):
      item_len_list = self.get_col_max_length(data_dict['values'], 
                                         data_dict['items'])
      data_dict_list[index]['lens'] = item_len_list
    return data_dict_list
         
  def create_excel_title(self, work_sheet, title, title_style=None):
    '''
    生产excel title
    '''
    if not title_style:
      title_style = self.title_style
    for col_index, col_name in enumerate(title):
      work_sheet.write(0, col_index, col_name, title_style)
    return work_sheet
  
  def create_excel_body(self, work_sheet, body, body_style=None):
    '''
    生成excel body信息
    '''
    if not body_style:
      body_style = self.default_style
    for row_num, row_data in enumerate(body, 1):
      for col_index, col_value in enumerate(row_data):
        try:
          col_value = float(col_value)
        except ValueError:
          pass
        work_sheet.write(row_num, col_index, col_value, body_style)
    return work_sheet
  
  def set_work_sheet_col_len(self, work_sheet, max_len):
    '''
    设置列长度
    '''
    for col, len in enumerate(max_len):
      work_sheet.col(col).width = 256 * (len + 1)
    return work_sheet
  
  def get_effect_line_list(self, file_name=None):
    '''
    过滤orzdba我们不需要的信息
    '''
    if file_name:
      self.file_name = file_name
    line_list = linecache.getlines(self.file_name)
    for num, line in enumerate(line_list, 0):
      if line.strip().startswith('time'):
        effect_line_list = line_list[num-1:len(line_list)-1]
        return effect_line_list

  def create_excel(self, data_dict_list):
    '''
    生成excel
    ''' 
    # 定义excel文件
    work_book = xlwt.Workbook(encoding='utf-8')
    for data_dict in data_dict_list:
      # 定义excel中的一个模板
      work_sheet = work_book.add_sheet(data_dict['module'])
      # 生成excel title
      work_sheet = self.create_excel_title(work_sheet,
                                      data_dict['items'])
      # 生成 excel 数据
      work_sheet = self.create_excel_body(work_sheet, 
                                          data_dict['values'])
      # 设置每一列适当的长度
      work_sheet = self.set_work_sheet_col_len(work_sheet, 
                                               data_dict['lens'])
    # 生成保存excel文件
    excel_name = r'{dir}/orzdba_report.xls'.format(
      dir  = self.pic_dir
    )
    work_book.save(excel_name)

  def create_picture(self, data_dict_list):
    '''
    创建图像
    '''
    # 导入数据分析和绘图包
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import numpy as np
    import pandas as pd

    for dict_index, data_dict in enumerate(data_dict_list, 0):
      # 以items为列名
      columns = data_dict['items']
      # 通过收集的数据和列名生成DataFrame对象
      df = pd.DataFrame(data_dict['values'], columns=columns)
      # 以time为索引
      df.index = df['time']
      del df['time']
      # 将数据转化成数字
      df = df.apply(pd.to_numeric, args=('coerce',))
      # 获得x坐标每个坐标展示的长度,以20的长度计算
      row_count = len(df.index)
      offset = row_count / 30 + 1
      # 画图
      df.plot(title   = data_dict['module'],
              figsize = (20, 10),
              xticks  = range(0, row_count, offset),
              grid    = True)
      # 设置x轴字倾斜
      locs, labels = plt.xticks()
      plt.setp(labels, rotation=60)
      # 保存图片
      plt.savefig(r'{pic_dir}/{file_name}.jpg'.format(
        pic_dir = self.pic_dir,
        file_name=data_dict['module']
      ))
    
if __name__ == '__main__':
  # 获取分析文件名
  if len(sys.argv) <> 2:
    print 'Usage: python orzdba_report.py logFileName'
    os._exit(1)
  file_name = sys.argv[1]
  orz_report = OrzReport(file_name)
  # 获取有效的收集的数据信息
  effect_line_list = orz_report.get_effect_line_list(file_name)
  # 获得每个指标的字符串列表
  item_str_list = orz_report.get_item_str_list(effect_line_list[1], 
                                               sep='|')
  # 获得分割字符串位置
  index_list = orz_report.get_split_index_list(effect_line_list[1], 
                                               sep='|')
  # 获得模块列表
  module_list = orz_report.get_module_list(effect_line_list[0], 
                                           index_list)
  # 合并模块和指标
  module_item_list = zip(module_list, item_str_list)
  # 获取格式化后的模块、指标名、指标值
  data_dict_list = orz_report.build_data_list(effect_line_list[2:], 
                                              module_item_list, sep='|')
  # 生成每个指标的长度
  data_dict_list = orz_report.create_item_len(data_dict_list)
  data_dict_list = orz_report.add_item_value_to_module(
                   data_dict_list[0],
                   data_dict_list[1:],
                   pos=0)
  # 生成excel
  orz_report.create_excel(data_dict_list)
  # 生成报表图片
  orz_report.create_picture(data_dict_list)
